import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Button } from 'antd'

import MKEmitter from '@/utils/events.js'
import './index.scss'

class StyleCombControlButton extends Component {
  static propTpyes = {
    menu: PropTypes.any
  }

  state = {
    label: '调整',
    parent: null,
    components: []
  }

  componentDidMount () {
    MKEmitter.addListener('clickComponent', this.clickComponent)
    MKEmitter.addListener('submitCombineStyle', this.submitCombineStyle)
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('clickComponent', this.clickComponent)
    MKEmitter.removeListener('submitCombineStyle', this.submitCombineStyle)
  }

  submitCombineStyle = (style) => {
    const { components } = this.state

    components.forEach((item, i) => {
      setTimeout(() => {
        MKEmitter.emit('submitStyle', [item.uuid], {...item.style, ...style})
      }, i * 5)
    })
  }

  clickComponent = (card) => {
    const { menu } = this.props
    const { parent } = this.state
    let components = fromJS(this.state.components).toJS()

    if (!parent) {
      let _parent = null
      let getParents = (box) => {
        box.components.forEach(item => {
          if (item.type === 'tabs') {
            item.subtabs.forEach(tab => {
              if (tab.components.findIndex(cell => cell.uuid === card.uuid) > -1) {
                _parent = tab
              } else {
                getParents(tab)
              }
            })
          } else if (item.type === 'group') {
            if (item.components.findIndex(cell => cell.uuid === card.uuid) > -1) {
              _parent = item
            }
          }
        })
      }

      if (menu.components.findIndex(cell => cell.uuid === card.uuid) > -1) {
        _parent = menu
      } else {
        getParents(menu)
      }

      document.getElementById(card.uuid).classList.add('selected-control-element')
      MKEmitter.emit('changeCombineStyle', ['margin', 'padding', 'border', 'background'])

      this.setState({
        parent: _parent,
        components: [card]
      })
    } else {
      if (components.findIndex(cell => cell.uuid === card.uuid) > -1) {
        components = components.filter(cell => cell.uuid !== card.uuid)
        document.getElementById(card.uuid).classList.remove('selected-control-element')
      } else if (parent.components.findIndex(cell => cell.uuid === card.uuid) > -1) {
        components.push(card)
        document.getElementById(card.uuid).classList.add('selected-control-element')
      }

      this.setState({
        components: components,
        parent: components.length ? parent : null
      })
    }
  }

  triggerStyleChange = () => {
    const { label } = this.state

    if (label === '调整') {
      document.body.className = 'style-control'
      sessionStorage.setItem('style-control', 'true')
      this.setState({label: '退出'})
    } else {
      document.body.className = ''
      sessionStorage.setItem('style-control', 'false')

      let eles = document.getElementsByClassName('selected-control-element')
      for (let i = 0; i < eles.length; i++) {
        eles[i].classList.remove('selected-control-element')
      }

      MKEmitter.emit('closeCombineStyle')

      this.setState({label: '调整', parent: null, components: []})
    }
  }
  
  render() {
    const { label } = this.state
    return (
      <Button className="style-control-button" type="link" icon="font-colors" title="调整样式" onClick={this.triggerStyleChange}>{label}</Button>
    )
  }
}

export default StyleCombControlButton